/* 
 *  Yacc grammar for the parser.  The files parser.mli and parser.ml
 *  are generated automatically from parser.mly.
 */

%{
open Support.Error
open Typ
open Exp
let dum = Typ.unknown_aspect
%}

%token <Support.Error.info> ARROW
%token <Support.Error.info> LINIMP
%token <Support.Error.info> COMMA
%token <Support.Error.info> BOX
%token <Support.Error.info> EOF
%token <Support.Error.info> EQ
%token <Support.Error.info> FUN
%token <string Support.Error.withinfo> ID
%token <Support.Error.info> IF
%token <Support.Error.info> IMPORT
%token <Support.Error.info> IN
%token <Support.Error.info> NAT
%token <int Support.Error.withinfo> NATV
%token <Support.Error.info> LET
%token <string Support.Error.withinfo> STRINGV
%token <Support.Error.info> LPAREN
%token <Support.Error.info> LSQUARE
%token <string Support.Error.withinfo> PRIMOP
%token <Support.Error.info> SEMI
%token <Support.Error.info> COMMA
%token <Support.Error.info> SZERO
%token <Support.Error.info> SONE
%token <Support.Error.info> SHOWTYPE
%token <Support.Error.info> THEN
%token <Support.Error.info> TYPE
%token <Support.Error.info> NOW
%token <Support.Error.info> SET
%token <Support.Error.info> RESET
%token <Support.Error.info> DO
%token <Support.Error.info> COLON
%token <Support.Error.info> RPAREN
%token <Support.Error.info> RSQUARE
%token <Support.Error.info> CASE
%token <Support.Error.info> VAL
%token <Support.Error.info> ZERO
%token <Support.Error.info> EVEN
%token <Support.Error.info> ODD
%token <Support.Error.info> SAFEREC
%token <Support.Error.info> LINREC

%start toplevel
%type <Exp.command list> toplevel

%%

toplevel :
  EOF                            { []     }
| Command toplevel               { $1::$2 }
;

Command :
  VAL Val                        { Val($2)                         }
| VAL ID EQ Val                  { ValBind($2.v,$4)                }
| IMPORT STRINGV                 { Import($2.v)                    }
| TYPE ID EQ Ty                  { AbbrevBind($1,$2.v,$4)          }
| NOW SET ID                     { Set($1,$3.v,true)               }
| NOW RESET ID                   { Set($1,$3.v,false)              }
| NOW DO ID                      { Do($1,$3.v,[])                  }
| SHOWTYPE ID                    { ShowType($1,$2.v)               }
;

   
Val :
  TVal                                     { $1               }
| TVal COLON Ty                            { Coerce($2,$1,$3) }
;

CaseSep : 
  ZERO                                     {0}
| EVEN                                     {1}
| ODD                                      {2}
;

TVal :
  FUN LPAREN ID COLON Ty RPAREN TVal       { Abs($1,$3.v,dum,$5,$7)}
| LET ID EQ Val IN TVal                    { Let($1,$2.v,dum,$4,$6)}
| CASE LSQUARE Ty RSQUARE Val CaseSep Val CaseSep Val CaseSep TVal    
                           { gencase($1,$3,$5,($6,$7),($8,$9),($10,$11)) }
| AppVal                                   { $1                          }
;

AppVal :
  AVal                           { $1                    }
| AppVal AVal                    { App(expInfo $1,$1,$2) }
;

AVal :
  ID                             { Var($1.i,$1.v)            }
| SZERO                          { SZero($1)                 }
| SONE                           { SOne($1)                  }
| SAFEREC LSQUARE Ty RSQUARE     { Saferec($1,$3)            }
| LINREC LSQUARE Ty RSQUARE      { Linrec($1,$3)             }
| LPAREN Val RPAREN              { $2                        }
| NATV                           { Con($1.i,$1.v)            }
;    

Ty :
  ATy LINIMP Ty                 { TArr((Linear,Nonmodal),$1,$3)       }
| ATy ARROW Ty                  { TArr((Nonlinear,Nonmodal),$1,$3)    }
| BOX ATy LINIMP Ty             { TArr((Linear,Modal),$2,$4)          }
| BOX ATy ARROW Ty              { TArr((Nonlinear,Modal),$2,$4)       }
| ATy                           { $1 };

ATy :
  ID                                      { TVar($1.v)          }
| NAT                                     { TNat                }
| LPAREN Ty RPAREN                        { $2                  }
;









