open Format
open Exp

type context = Eval.context * Typcontext.ctx

let initial_context = (Eval.empty_context, Typcontext.empty)

let show = 
  Support.Flags.createFlag "show"
    "Show the results of top-level typechecking and evaluation"
    true

let do_command ((ectx,tctx) as ctx) = function
    Val(e) -> 
      let (_,e,ty) = Check.check tctx e in
      let v = Eval.eval ectx e in
      if !show then (
        Eval.print v; print_string " : "; 
        Typ.print (Conv.deep_expose tctx ty);
        force_newline());
      ctx
  | ValBind(x,e) -> 
      let (_,e,ty) = Check.check tctx e in
      let v = Eval.eval ectx e in
      if !show then (
        print_string x;
        print_string " : "; Typ.print (Conv.deep_expose tctx ty);
        force_newline();
        print_string (" "^x^" = ");Eval.print v;
        force_newline());
      (Eval.bind x v Typ.unknown_aspect  ectx, Typcontext.bind x ty 
                 Typ.unknown_aspect (Typcontext.remove_var x tctx))
  | ShowType(fi,x) -> 
      print_string x; print_string " : ";
      Typ.print (Conv.deep_expose tctx (Typcontext.extract_type x tctx));
      force_newline();
      (ectx, tctx)
  | AbbrevBind(fi,x,ty) -> 
      if !show then (
        print_string x; print_string " = ";
        Typ.print (Conv.deep_expose tctx ty); 
        force_newline());
      (ectx, Typcontext.abbrevbind x ty tctx)
  | _ ->
      (* Remaining cases are handled elsewhere *)
      Support.Error.bug "Toplevel.do_command"


